<?php
session_start();
if (!isset($_SESSION['user'])) {
    header("Location: ../login.php");
    exit;
}

// 引入数据库配置文件
require_once '../config.php';

// 数据库连接
$conn = get_db_connection();

// 获取商品列表
$products = [];
$productResult = $conn->query("SELECT id, product_code, product_name, unit, multi_unit FROM products ORDER BY product_code");
if ($productResult) {
    while ($row = $productResult->fetch_assoc()) {
        $products[] = $row;
    }
}

// 获取仓库列表
$warehouses = [];
$warehouseResult = $conn->query("SELECT id, name FROM warehouses ORDER BY name");
if ($warehouseResult) {
    while ($row = $warehouseResult->fetch_assoc()) {
        $warehouses[] = $row;
    }
}

// 保存用户选择的批量入库状态
$bulk_mode = isset($_POST['bulk_mode']) ? (bool)$_POST['bulk_mode'] : false;
$selected_product = '';
$selected_warehouse = '';

// 处理表单提交
$message = '';
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $product_id = isset($_POST['product_id']) ? intval($_POST['product_id']) : 0;
    $selected_product = $product_id;

    // 获取商品名称
    $product_name = '';
    foreach ($products as $product) {
        if ($product['id'] == $product_id) {
            $product_name = $product['product_name'];
            break;
        }
    }

    if ($product_id <= 0) {
        $message = '❌ 请选择商品!';
    } else {
        $hasEntries = false;
        $allEmpty = true;

        // 开始事务
        $conn->begin_transaction();

        try {
            // 处理每个仓库的入库数量
            foreach ($warehouses as $warehouse) {
                $warehouse_id = $warehouse['id'];
                $quantity_key = "quantity_" . $warehouse_id;
                $unit_key = "unit_type_" . $warehouse_id;

                // 获取数量和单位类型
                $quantity = isset($_POST[$quantity_key]) && $_POST[$quantity_key] !== '' ? floatval($_POST[$quantity_key]) : 0;
                $unit_type = isset($_POST[$unit_key]) ? $_POST[$unit_key] : 'base';

                // 跳过数量为0的入库记录
                if ($quantity <= 0) {
                    continue;
                }

                $allEmpty = false;
                $hasEntries = true;

                // 获取基本单位数量
                $base_quantity = $quantity;

                // 如果不是基本单位，需要进行换算
                if ($unit_type !== 'base') {
                    // 查询单位换算比例
                    $ratioSql = "SELECT ratio FROM product_units WHERE product_id = ? AND unit_name = ? AND unit_type = 'sub'";
                    $ratioStmt = $conn->prepare($ratioSql);
                    $ratioStmt->bind_param("is", $product_id, $unit_type);
                    $ratioStmt->execute();
                    $ratioResult = $ratioStmt->get_result();

                    if ($ratioResult->num_rows > 0) {
                        $ratio = $ratioResult->fetch_assoc()['ratio'];
                        $base_quantity = $quantity * $ratio;
                    }
                }

                // 检查该商品在该仓库是否已有库存记录
                $checkSql = "SELECT id, base_quantity FROM inventory_stock WHERE product_id = ? AND warehouse_id = ?";
                $checkStmt = $conn->prepare($checkSql);
                $checkStmt->bind_param("ii", $product_id, $warehouse_id);
                $checkStmt->execute();
                $checkResult = $checkStmt->get_result();

                if ($checkResult->num_rows > 0) {
                    // 更新现有库存
                    $row = $checkResult->fetch_assoc();
                    $newQuantity = $row['base_quantity'] + $base_quantity;

                    $updateSql = "UPDATE inventory_stock SET base_quantity = ?, updated_by = ?, last_updated = NOW() WHERE id = ?";
                    $updateStmt = $conn->prepare($updateSql);
                    $userId = $_SESSION['user']['id'];
                    $updateStmt->bind_param("dii", $newQuantity, $userId, $row['id']);
                    $updateStmt->execute();
                } else {
                    // 新增库存记录
                    $insertSql = "INSERT INTO inventory_stock (product_id, warehouse_id, base_quantity, updated_by) VALUES (?, ?, ?, ?)";
                    $insertStmt = $conn->prepare($insertSql);
                    $userId = $_SESSION['user']['id'];
                    $insertStmt->bind_param("iidi", $product_id, $warehouse_id, $base_quantity, $userId);
                    $insertStmt->execute();
                }
            }

            $conn->commit();

            if ($allEmpty) {
                $message = '✅ 商品入库处理完成';

                // 如果不是批量模式，则跳转到库存列表页
                if (!$bulk_mode) {
                    $conn->close();
                    header("Location: inventory_stock.php?message=" . urlencode($message) . "&refresh=1");
                    exit;
                } else {
                    // 在批量模式下，清空选择的商品
                    $selected_product = '';
                }
            } else if ($hasEntries) {
                $message = '✅ 库存入库成功！';

                // 如果不是批量模式，则跳转到库存列表页
                if (!$bulk_mode) {
                    $conn->close();
                    header("Location: inventory_stock.php?message=" . urlencode($message) . "&refresh=1");
                    exit;
                } else {
                    // 在批量模式下，清空选择的商品
                    $selected_product = '';
                }
            }
        } catch (Exception $e) {
            $conn->rollback();
            $message = '❌ 入库失败：' . $e->getMessage();
        }
    }
}

$conn->close();

// 定义一个变量，用于在页面加载完成后执行JavaScript操作
$jsCommand = '';
if ($_SERVER['REQUEST_METHOD'] === 'POST' && !empty($message) && strpos($message, '✅') === 0 && $bulk_mode) {
    // 在批量模式下提交成功时，添加JavaScript命令
    $jsCommand = "
        // 显示成功消息
        if (document.getElementById('message-text')) {
            document.getElementById('message-text').innerHTML = '" . addslashes($message) . "';
            document.getElementById('message-container').style.display = 'flex';
        }
        
        // 使用AJAX清除缓存
        fetch('clear_inventory_cache.php')
            .then(response => response.json())
            .then(data => {
                console.log('Redis缓存清理结果:', data);
                
                // 如果缓存成功清除，创建一个隐藏的iframe来刷新库存列表
                if (data.success) {
                    // 先访问库存列表页面并强制刷新缓存
                    const refreshFrame = document.createElement('iframe');
                    refreshFrame.style.display = 'none';
                    refreshFrame.src = 'inventory_stock.php?refresh=1&_t=' + new Date().getTime();
                    document.body.appendChild(refreshFrame);
                    
                    // 2秒后移除iframe以避免内存泄漏
                    setTimeout(() => {
                        if (refreshFrame && refreshFrame.parentNode) {
                            refreshFrame.parentNode.removeChild(refreshFrame);
                        }
                    }, 2000);
                }
            })
            .catch(error => console.error('清除缓存失败:', error));
        
        // 重置表单并保持批量模式
        setTimeout(function() {
            // 重置商品选择
            document.getElementById('product-search').value = '';
            document.getElementById('product_id').value = '';
            
            // 重置所有数量输入框
            document.querySelectorAll('.quantity-input').forEach(input => {
                input.value = '';
            });
            
            // 滚动到页面顶部
            window.scrollTo({
                top: 0,
                behavior: 'smooth'
            });
        }, 1500);
    ";
}
?>
<!DOCTYPE html>
<html lang="zh">

<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>商品入库</title>
    <link rel="stylesheet" href="style.css">
    <link rel="stylesheet" href="./css/inventory_in_form.css">
</head>

<body>
    <div class="header">
        <h2>商品入库</h2>
        <a href="inventory_stock.php" class="btn btn-secondary">返回库存列表</a>
    </div>

    <!-- 消息显示 -->
    <?php if (!empty($message)): ?>
        <div id="message-container" class="message" style="display: flex;">
            <span id="message-text"><?= $message ?></span>
            <button onclick="document.getElementById('message-container').style.display='none'">×</button>
        </div>
    <?php endif; ?>

    <div class="container">
        <form action="" method="post" id="stock-form">
            <div class="form-row">
                <div class="form-col" style="flex-basis: 100%">
                    <label>选择商品：</label>
                    <div class="search-select-container">
                        <input type="text" id="product-search" class="search-input" placeholder="点击选择或输入搜索商品" autocomplete="off">
                        <input type="hidden" id="product_id" name="product_id" value="<?= $selected_product ?>" required>
                        <div id="search-results" class="search-results"></div>
                    </div>
                    <small>共 <?= count($products) ?> 个商品</small>
                </div>
            </div>

            <div class="form-row" style="margin-top: 20px;">
                <div class="form-col">
                    <label>&nbsp;</label>
                    <div class="bulk-mode-container">
                        <input type="checkbox" id="bulk_mode" name="bulk_mode" value="1" <?= $bulk_mode ? 'checked' : '' ?>>
                        <label for="bulk_mode">批量入库</label>
                    </div>
                    <div style="padding-top: 8px;">
                        <small id="quantity-note">注意：可选择不同单位输入，系统会自动换算为基本单位</small>
                    </div>
                </div>
            </div>

            <h3 style="margin-top: 20px;">入库数量设置</h3>

            <table class="inventory-table" style="margin-top: 10px;">
                <thead>
                    <tr>
                        <th style="width: 30%;">仓库名称</th>
                        <th style="width: 30%;">单位选择</th>
                        <th style="width: 40%;">入库数量</th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($warehouses as $warehouse): ?>
                        <tr>
                            <td><?= htmlspecialchars($warehouse['name']) ?></td>
                            <td>
                                <select name="unit_type_<?= $warehouse['id'] ?>"
                                    id="unit_type_<?= $warehouse['id'] ?>"
                                    class="unit-select"
                                    data-warehouse-id="<?= $warehouse['id'] ?>">
                                    <option value="base">基本单位</option>
                                    <!-- 副单位将通过JavaScript动态添加 -->
                                </select>
                            </td>
                            <td>
                                <input type="number" name="quantity_<?= $warehouse['id'] ?>"
                                    id="quantity_<?= $warehouse['id'] ?>"
                                    min="0" step="1"
                                    placeholder="请输入入库数量（可为0）"
                                    class="quantity-input">
                            </td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>

            <div class="btn-group">
                <button type="submit" class="btn btn-success">提交入库</button>
                <button type="button" class="btn btn-secondary" onclick="window.location.href='inventory_stock.php'">取消</button>
            </div>
        </form>
    </div>

    <script>
        // 商品数据
        const products = <?= json_encode($products) ?>;

        // 商品搜索功能
        const productSearch = document.getElementById('product-search');
        const searchResults = document.getElementById('search-results');
        const productIdInput = document.getElementById('product_id');
        let selectedProduct = null;
        let productUnits = [];

        // 页面加载时，如果已有选中的商品ID，则初始化商品显示
        document.addEventListener('DOMContentLoaded', function() {
            const selectedProductId = '<?= $selected_product ?>';
            if (selectedProductId) {
                const product = products.find(p => p.id == selectedProductId);
                if (product) {
                    const productData = {
                        id: product.id,
                        name: product.product_name,
                        code: product.product_code,
                        unit: product.unit,
                        multiUnit: product.multi_unit
                    };
                    selectProduct(productData);
                }
            }
        });

        // 点击商品输入框显示所有商品
        productSearch.addEventListener('click', function() {
            if (searchResults.style.display !== 'block') {
                showAllProducts();
            }
        });

        // 搜索框获得焦点时显示结果
        productSearch.addEventListener('focus', function() {
            if (searchResults.style.display !== 'block') {
                showAllProducts();
            }
        });

        // 显示所有商品
        function showAllProducts() {
            searchResults.innerHTML = '';
            // 按ID降序排序，使新添加的商品（ID较大的）显示在上方
            const sortedProducts = [...products].sort((a, b) => b.id - a.id);
            sortedProducts.forEach(product => {
                const item = document.createElement('div');
                item.className = 'search-result-item';
                item.textContent = `${product.product_name} (${product.product_code})`;
                item.dataset.id = product.id;
                item.dataset.name = product.product_name;
                item.dataset.code = product.product_code;
                item.dataset.unit = product.unit;
                item.dataset.multiUnit = product.multi_unit;

                item.addEventListener('click', function() {
                    selectProduct(this.dataset);
                });

                searchResults.appendChild(item);
            });

            searchResults.style.display = 'block';
        }

        // 搜索商品
        productSearch.addEventListener('input', function() {
            const searchTerm = this.value.toLowerCase();
            if (searchTerm.length < 1) {
                showAllProducts();
                return;
            }

            // 过滤匹配的商品
            const filteredProducts = products.filter(p =>
                p.product_name.toLowerCase().includes(searchTerm) ||
                p.product_code.toLowerCase().includes(searchTerm)
            );

            // 按ID降序排序过滤结果
            filteredProducts.sort((a, b) => b.id - a.id);

            // 显示搜索结果
            searchResults.innerHTML = '';
            filteredProducts.forEach(product => {
                const item = document.createElement('div');
                item.className = 'search-result-item';
                item.textContent = `${product.product_name} (${product.product_code})`;
                item.dataset.id = product.id;
                item.dataset.name = product.product_name;
                item.dataset.code = product.product_code;
                item.dataset.unit = product.unit;
                item.dataset.multiUnit = product.multi_unit;

                item.addEventListener('click', function() {
                    selectProduct(this.dataset);
                });

                searchResults.appendChild(item);
            });

            searchResults.style.display = filteredProducts.length > 0 ? 'block' : 'none';
        });

        // 点击外部关闭搜索结果
        document.addEventListener('click', function(e) {
            if (e.target !== productSearch && e.target !== searchResults && !searchResults.contains(e.target)) {
                searchResults.style.display = 'none';
            }
        });

        // 选择商品
        function selectProduct(productData) {
            selectedProduct = productData;
            productSearch.value = `${productData.name} (${productData.code})`;
            productIdInput.value = productData.id;
            searchResults.style.display = 'none';

            // 清除所有单位选择器
            document.querySelectorAll('.unit-select').forEach(select => {
                select.innerHTML = '';
                const baseOption = document.createElement('option');
                baseOption.value = 'base';
                baseOption.textContent = productData.unit;
                select.appendChild(baseOption);
            });

            // 如果是多单位商品，获取副单位
            if (productData.multiUnit === '1') {
                fetchProductUnits(productData.id);
            } else {
                productUnits = [];
            }
        }

        // 获取商品单位
        function fetchProductUnits(productId) {
            fetch(`get_product_units.php?product_id=${productId}`)
                .then(response => response.json())
                .then(data => {
                    productUnits = data;

                    // 添加副单位到所有选择框
                    document.querySelectorAll('.unit-select').forEach(select => {
                        data.forEach(unit => {
                            if (unit.unit_type === 'sub') {
                                const option = document.createElement('option');
                                option.value = unit.unit_name;
                                option.textContent = unit.unit_name;
                                select.appendChild(option);
                            }
                        });
                    });
                })
                .catch(error => {
                    console.error('获取商品单位失败:', error);
                });
        }

        // 自动关闭消息提示
        setTimeout(() => {
            const messageBox = document.getElementById('message-container');
            if (messageBox) {
                messageBox.style.display = 'none';
            }
        }, 3000);

        // 检查是否为批量入库模式且表单提交后需要清空
        const bulkMode = <?= json_encode($bulk_mode) ?>;
        const formSubmitted = <?= json_encode(!empty($message) && (strpos($message, '✅') === 0)) ?>;

        if (bulkMode && formSubmitted) {
            // 清空商品选择
            productSearch.value = '';
            productIdInput.value = '';
            selectedProduct = null;

            // 重置所有数量输入框
            document.querySelectorAll('.quantity-input').forEach(input => {
                input.value = '';
            });
        }

        // Redis缓存调试信息
        <?php if (isset($redisManager) && $redisManager->isAvailable()): ?>
            console.log('========== Redis缓存操作调试信息 ==========');
            console.log('Redis可用: 是');
            console.log('Redis前缀: <?= isset($redisPrefix) ? $redisPrefix : ($redis_prefix ?? "无") ?>');
            console.log('操作类型: <?= $bulk_mode ? "批量入库模式" : "单次入库模式" ?>');
            <?php if (isset($deletedCount)): ?>
                console.log('缓存键模式: inventory:product:*');
                console.log('已删除缓存数量: <?= $deletedCount ?>');
                <?php
                // 输出所有Redis相关键
                $allKeys = $redisManager->getRedis()->keys('*');
                if (!empty($allKeys)):
                ?>
                    console.log('当前Redis中与库存相关的键:');
                    <?php
                    $inventoryKeys = array_filter($allKeys, function ($key) {
                        return strpos($key, 'inventory') !== false;
                    });
                    foreach (array_slice($inventoryKeys, 0, 10) as $key):
                    ?>
                        console.log(' - <?= $key ?>');
                    <?php endforeach; ?>
                    <?php if (count($inventoryKeys) > 10): ?>
                        console.log('...更多键未显示 (总计<?= count($inventoryKeys) ?>个)');
                    <?php elseif (count($inventoryKeys) == 0): ?>
                        console.log('没有找到与库存相关的缓存键');
                    <?php endif; ?>
                <?php endif; ?>
            <?php endif; ?>
            console.log('========================================');
        <?php endif; ?>

        // 在页面加载完成后执行批量模式的表单重置和缓存清理
        document.addEventListener('DOMContentLoaded', function() {
            <?php if (!empty($jsCommand)): ?>
                <?= $jsCommand ?>
            <?php endif; ?>
        });
    </script>
</body>

</html>